package com.thesis.backend.main;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import cc.mallet.types.InstanceList;

import com.thesis.backend.classifiers.CRF_Classifier;
import com.thesis.backend.classifiers.MaxEnt_Classifier;
import com.thesis.backend.entity.Paper;
import com.thesis.backend.util.IOOperations;
import com.thesis.backend.util.PreProcessor;

/**
 * Main class for the backend. Access private methods in classifiers through this class.  
 *
 */
public class Main {

	// Filepath to stored and trained models
	private static final String cl_model = "bin\\Article_Sys\\Models\\maxEnt.mallet";
	private static final String crf_model = "bin\\Article_Sys\\Models\\crf.mallet";

	private CRF_Classifier crf;
	private MaxEnt_Classifier maxEnt;

	private PreProcessor process = new PreProcessor();
	private Paper paper;

	/**
	 * Load all models into the system
	 */
	public Main() {
		try {
			crf = new CRF_Classifier(crf_model);
			maxEnt = new MaxEnt_Classifier(cl_model);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Send paper object through the models to get values
	 * @param file
	 * @return paper
	 * @throws IOException
	 */
	public Paper startPaperJourney(File file) throws IOException {
		System.out.println("FILEPATH IS: " + file.getAbsolutePath());
		String raw = IOOperations.readFromFile(file.getAbsolutePath());
		paper = new Paper(raw, false, file.getName());
		process.parseToMalletFormats(paper);
		runModels();
		return paper;
	}

	/**
	 * Run MaxEnt and CRF classifiers
	 * @throws IOException
	 */
	private void runModels() throws IOException {
		InstanceList crf_instance = crf
				.putThroughPipes(paper.getCrf_ML_input());
		crf.runCrf(crf_instance.get(0), paper);

		InstanceList doc_instance = maxEnt.putThroughPipes(paper
				.getCl_ML_input());
		String value = maxEnt.runMaxEnt(doc_instance.get(0));
		paper.setApproach(value);
	}

	/**
	 * For testing, validation and loading new models into the backend
	 * @param args
	 */
	public static void main(String args[]) {
		Main main = new Main();
		Paper paper;
		try {
			paper = main.startPaperJourney(new File("bin/test_data/test1.txt"));
			System.out.println(paper.toString());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
